// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#ifndef _FreeOTFEDebug_H
#define _FreeOTFEDebug_H   1


//#define DBG 0


// =========================================================================
// Debug related

#if DBG

extern ULONG FreeOTFEDebugLevel;
extern ULONG FreeOTFEDebugIndent;
extern ULONG FreeOTFEDebugLoopTmp;

// For checked kernels, define a macro to print out informational
// messages.
//
// Used for errors
#define DEBUGLEV_ERROR             ((ULONG)0x00000001)
// Used for warnings
#define DEBUGLEV_WARN              ((ULONG)0x00000002)
// Used for information
#define DEBUGLEV_INFO              ((ULONG)0x00000004)
// Used when entering a function
#define DEBUGLEV_ENTER             ((ULONG)0x00000008)
// Used when exiting a function
#define DEBUGLEV_EXIT              ((ULONG)0x00000010)

// Used for errors
#define DEBUGLEV_VERBOSE_ERROR     ((ULONG)0x00001000)
// Used for warnings
#define DEBUGLEV_VERBOSE_WARN      ((ULONG)0x00002000)
// Used for information
#define DEBUGLEV_VERBOSE_INFO      ((ULONG)0x00004000)
// Used when entering a function
#define DEBUGLEV_VERBOSE_ENTER     ((ULONG)0x00008000)
// Used when exiting a function
#define DEBUGLEV_VERBOSE_EXIT      ((ULONG)0x00010000)


#define DEBUGOUT(DRIVER,LEVEL,STRING)                \
            if (FreeOTFEDebugLevel & (LEVEL))        \
                {                                    \
                DbgPrint DRIVER;                     \
                DbgPrint(": ");                      \
                if (                                 \
                    (LEVEL & DEBUGLEV_VERBOSE_ERROR) ||  \
                    (LEVEL & DEBUGLEV_VERBOSE_WARN)  ||  \
                    (LEVEL & DEBUGLEV_VERBOSE_INFO)  ||  \
                    (LEVEL & DEBUGLEV_VERBOSE_ENTER) ||  \
                    (LEVEL & DEBUGLEV_VERBOSE_EXIT)  \
                   )                                 \
                    {                                \
                    DbgPrint("(v)");                 \
                    }                                \
                if (                                 \
                    (LEVEL & DEBUGLEV_EXIT) ||       \
                    (LEVEL & DEBUGLEV_VERBOSE_EXIT)  \
                   )                                 \
                    {                                \
                    FreeOTFEDebugIndent--;           \
                    }                                \
                for (FreeOTFEDebugLoopTmp = 0;       \
                     FreeOTFEDebugLoopTmp<=FreeOTFEDebugIndent;  \
                     FreeOTFEDebugLoopTmp++)         \
                    {                                \
                    DbgPrint("  ");                  \
                    }                                \
                if (                                 \
                    (LEVEL & DEBUGLEV_ERROR) ||      \
                    (LEVEL & DEBUGLEV_VERBOSE_ERROR) \
                   )                                 \
                    {                                \
                    DbgPrint("ERROR: ");             \
                    }                                \
                else if (                            \
                    (LEVEL & DEBUGLEV_WARN) ||       \
                    (LEVEL & DEBUGLEV_VERBOSE_WARN)  \
                   )                                 \
                    {                                \
                    DbgPrint("WARNING: ");           \
                    }                                \
                else if (                            \
                    (LEVEL & DEBUGLEV_ENTER) ||      \
                    (LEVEL & DEBUGLEV_VERBOSE_ENTER) \
                   )                                 \
                    {                                \
                    DbgPrint(">>> ");                \
                    FreeOTFEDebugIndent++;           \
                    }                                \
                else if (                            \
                    (LEVEL & DEBUGLEV_EXIT) ||       \
                    (LEVEL & DEBUGLEV_VERBOSE_EXIT)  \
                   )                                 \
                    {                                \
                    DbgPrint("<<< ");                \
                    }                                \
                                                     \
                DbgPrint STRING;                     \
                }

#define DEBUGOUTMAINDRV(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFE"), LEVEL, STRING)

//#define DEBUGOUTMACDRV(LEVEL,STRING) \
//                DEBUGOUT(("FreeOTFEMAC"), LEVEL, STRING)
#define DEBUGOUTMACDRV(LEVEL,STRING)

//#define DEBUGOUTKDFDRV(LEVEL,STRING) \
//                DEBUGOUT(("FreeOTFEKDF"), LEVEL, STRING)
#define DEBUGOUTKDFDRV(LEVEL,STRING)

#define DEBUGOUTLIB(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFElib"), LEVEL, STRING)

#define DEBUGOUTCYPHERDRV(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFECypherDrv"), LEVEL, STRING)

#define DEBUGOUTCYPHERIMPL(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFECypherImpl"), LEVEL, STRING)

#define DEBUGOUTHASHDRV(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFEHashDrv"), LEVEL, STRING)

#define DEBUGOUTHASHIMPL(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFEHashImpl"), LEVEL, STRING)

#define DEBUGOUTIVDRV(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFEIVDrv"), LEVEL, STRING)

#define DEBUGOUTIVIMPL(LEVEL,STRING) \
                DEBUGOUT(("FreeOTFEIVImpl"), LEVEL, STRING)

#else
#define DEBUGOUT(DRIVER,LEVEL,STRING) 

#define DEBUGOUTMAINDRV(LEVEL,STRING) 
#define DEBUGOUTMACDRV(LEVEL,STRING)
#define DEBUGOUTKDFDRV(LEVEL,STRING)
#define DEBUGOUTLIB(LEVEL,STRING) 
#define DEBUGOUTCYPHERDRV(LEVEL,STRING) 
#define DEBUGOUTCYPHERIMPL(LEVEL,STRING)
#define DEBUGOUTHASHDRV(LEVEL,STRING) 
#define DEBUGOUTHASHIMPL(LEVEL,STRING) 
#define DEBUGOUTIVDRV(LEVEL,STRING) 
#define DEBUGOUTIVIMPL(LEVEL,STRING)

#endif



// =========================================================================
// =========================================================================

#endif

