// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFEHashImpl.h"
#include "FreeOTFEHashSHA.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// Include TomLibCrypt libraries...
#include <mycrypt_custom.h>


// =========================================================================
// Hash driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpHashDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_HASH_VERSION;


    // -- POPULATE HASHES SUPPORTED --
    devExt->HashDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(HASH) * HASHES_SUPPORTED)
                                        );    

    devExt->HashCount = HASHES_SUPPORTED;


    // -- SHA-1 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_SHA1,
                  strlen(DRIVER_HASH_TITLE_SHA1)
                 );

    devExt->HashDetails[idx].Length = (sha1_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (sha1_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_SHA1;


    // -- SHA-224 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_SHA224,
                  strlen(DRIVER_HASH_TITLE_SHA224)
                 );

    devExt->HashDetails[idx].Length = (sha224_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (sha224_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_SHA224;


    // -- SHA-256 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_SHA256,
                  strlen(DRIVER_HASH_TITLE_SHA256)
                 );

    devExt->HashDetails[idx].Length = (sha256_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (sha256_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_SHA256;


    // -- SHA-384 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_SHA384,
                  strlen(DRIVER_HASH_TITLE_SHA384)
                 );

    devExt->HashDetails[idx].Length = (sha384_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (sha384_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_SHA384;


    // -- SHA-512 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_SHA512,
                  strlen(DRIVER_HASH_TITLE_SHA512)
                 );

    devExt->HashDetails[idx].Length = (sha512_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (sha512_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_SHA512;



    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Hash driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpHashDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsCleardown\n"));

    if (devExt->HashDetails != NULL)
        {
        ExFreePool(devExt->HashDetails);
        }

    devExt->HashDetails = NULL;
    devExt->HashCount = 0;

    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Hash function
NTSTATUS
ImpHashHashData(
    IN      GUID* HashGUID,
    IN      unsigned int DataLength,  // In bits
    IN      unsigned char* Data,
    IN OUT  unsigned int* HashLength,  // In bits
    OUT     unsigned char* Hash
)
{
    NTSTATUS status = STATUS_SUCCESS;
    hash_state md;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashHashData\n"));
    if (IsEqualGUID(&HASH_GUID_SHA512, HashGUID))
        {
        if (*HashLength < (sha512_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (sha512_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            sha512_desc.init(&md);
            sha512_desc.process(&md, Data, (DataLength / 8));
            sha512_desc.done(&md, Hash);
        
            *HashLength = (sha512_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_SHA384, HashGUID))
        {
        if (*HashLength < (sha384_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (sha384_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            sha384_desc.init(&md);
            sha384_desc.process(&md, Data, (DataLength / 8));
            sha384_desc.done(&md, Hash);
        
            *HashLength = (sha384_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_SHA256, HashGUID))
        {
        if (*HashLength < (sha256_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (sha256_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            sha256_desc.init(&md);
            sha256_desc.process(&md, Data, (DataLength / 8));
            sha256_desc.done(&md, Hash);
        
            *HashLength = (sha256_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_SHA224, HashGUID))
        {
        if (*HashLength < (sha224_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (sha224_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            sha224_desc.init(&md);
            sha224_desc.process(&md, Data, (DataLength / 8));
            sha224_desc.done(&md, Hash);
        
            *HashLength = (sha224_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_SHA1, HashGUID))
        {
        if (*HashLength < (sha1_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (sha1_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            sha1_desc.init(&md);
            sha1_desc.process(&md, Data, (DataLength / 8));
            sha1_desc.done(&md, Hash);
        
            *HashLength = (sha1_desc.hashsize * 8);
            }
            
        }
    else
        {
        DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("driver doesn't recognise GUID\n"));
        status = STATUS_INVALID_PARAMETER;
        }
        
    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashHashData\n"));
      
    return status;
}


// =========================================================================
// =========================================================================

